@@ -1,16 +1,24 @@
-use crate::fuzzer::{self, Fuzzer};
-use crate::traits::{Mutator, Serializer};
+use crate::code_coverage_sensor::CodeCoverageSensor;
+use crate::fuzzer::{Fuzzer, ReasonForStopping};
+use crate::sensors_and_pools::and_sensor_and_pool::AndPool;
+use crate::sensors_and_pools::maximize_pool::CounterMaximizingPool;
+use crate::sensors_and_pools::sum_coverage_pool::{
+    AggregateCoveragePool, CountNumberOfDifferentCounters, SumCounterValues,
+};
+use crate::sensors_and_pools::unique_coverage_pool::UniqueCoveragePool;
+use crate::traits::{CompatibleWithSensor, Mutator, Pool, Sensor, Serializer, TestCase};
+use crate::{DefaultMutator, FuzzedInput, SerdeSerializer};
 
-use fuzzcheck_common::arg::Arguments;
 use fuzzcheck_common::arg::{
     options_parser, COMMAND_FUZZ, COMMAND_MINIFY_CORPUS, COMMAND_MINIFY_INPUT, CORPUS_SIZE_FLAG, INPUT_FILE_FLAG,
     IN_CORPUS_FLAG,
 };
-
+use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
 use std::borrow::Borrow;
 use std::marker::PhantomData;
 use std::path::Path;
 use std::result::Result;
+use std::time::Duration;
 
 /** A function that can be fuzz-tested.
 
@@ -36,7 +44,7 @@ where
     F: Fn(&T) -> bool,
 {
     type NormalizedFunction = Self;
-
+    #[no_coverage]
     fn test_function(self) -> Self::NormalizedFunction {
         self
     }
@@ -46,7 +54,7 @@ where
     F: Fn(&T),
 {
     type NormalizedFunction = impl Fn(&T) -> bool;
-
+    #[no_coverage]
     fn test_function(self) -> Self::NormalizedFunction {
         move |x| {
             self(x);
@@ -60,51 +68,11 @@ where
     F: Fn(&T) -> Result<E, S>,
 {
     type NormalizedFunction = impl Fn(&T) -> bool;
-
+    #[no_coverage]
     fn test_function(self) -> Self::NormalizedFunction {
         move |x| self(x).is_ok()
     }
 }
-
-/**
-    Use this builder type to construct a fuzz test and launch it.
-
-    A fuzz-test is constructed by passing these five arguments, in order:
-
-    1. the function to fuzz-test
-    2. the mutator that produces the test cases
-    3. the serializer to use when saving the interesting test cases to the file system
-    4. other fuzzing arguments, which may be produced by `cargo-fuzzcheck`, or specified manually
-    5. the files whose code coverage influece the fuzzer
-
-    For example, you may write:
-    ```
-    #![feature(no_coverage)]
-    use fuzzcheck::{FuzzerBuilder, DefaultMutator, SerdeSerializer};
-
-    fn my_function(xs: &Option<u16>) -> bool {
-        // ..
-        # false
-    }
-    fn fuzz_test() {
-        FuzzerBuilder::test(my_function)
-            .mutator(<Option<u16>>::default_mutator())
-            .serializer(SerdeSerializer::default())
-            .arguments_from_cargo_fuzzcheck()
-            .observe_only_files_from_current_dir()
-            .launch();
-    }
-    ```
-
-    Each step is performed on a different type. You start with a
-    `FuzzerBuilder`, which asks for a test function. Once that test function
-    is given, you get a `FuzzerBuilder1`, which asks for a mutator.
-    `FuzzerBuilder2` asks for a serializer. `FuzzerBuilder3` asks for fuzzing
-    arguments. And finally, `FuzzerBuilder4` has all the information needed to
-    launch the fuzz test.
-*/
-pub enum FuzzerBuilder {}
-
 pub struct FuzzerBuilder1<T, F>
 where
     T: ?Sized,
@@ -139,62 +107,106 @@ where
     serializer: S,
     _phantom: PhantomData<(*const T, V)>,
 }
-
-pub struct FuzzerBuilder4<T, F, M, V, S>
+pub struct FuzzerBuilder4<T, F, M, V, S, Sens>
 where
     T: ?Sized,
     F: Fn(&T) -> bool,
     V: Clone + Borrow<T>,
     M: Mutator<V>,
     S: Serializer<Value = V>,
+    Sens: Sensor,
 {
     test_function: F,
     mutator: M,
     serializer: S,
-    arguments: Arguments,
+    sensor: Sens,
     _phantom: PhantomData<(*const T, V)>,
 }
-pub struct FuzzerBuilder5<T, F, M, V, S, Exclude, Keep>
+pub struct FuzzerBuilder5<T, F, M, V, S, Sens, P>
 where
     T: ?Sized,
     F: Fn(&T) -> bool,
     V: Clone + Borrow<T>,
     M: Mutator<V>,
     S: Serializer<Value = V>,
-    Exclude: Fn(&Path) -> bool,
-    Keep: Fn(&Path) -> bool,
+    Sens: Sensor,
+    P: Pool<TestCase = FuzzedInput<V, M>> + CompatibleWithSensor<Sens>,
 {
     test_function: F,
     mutator: M,
     serializer: S,
+    sensor: Sens,
+    pool: P,
+    _phantom: PhantomData<(*const T, V)>,
+}
+pub struct FuzzerBuilder6<T, F, M, V, S, Sens, P>
+where
+    T: ?Sized,
+    F: Fn(&T) -> bool,
+    V: Clone + Borrow<T>,
+    M: Mutator<V>,
+    S: Serializer<Value = V>,
+    Sens: Sensor,
+    P: Pool<TestCase = FuzzedInput<V, M>> + CompatibleWithSensor<Sens>,
+{
+    test_function: F,
+    mutator: M,
+    serializer: S,
+    sensor: Sens,
+    pool: P,
     arguments: Arguments,
-    exclude: Exclude,
-    keep: Keep,
     _phantom: PhantomData<(*const T, V)>,
 }
 
-impl FuzzerBuilder {
-    /**
-        Specify the function to fuzz-test.
+/**
+    Specify the function to fuzz-test.
 
-        There are currently three kinds of functions that can be passed as arguments:
+    There are currently three kinds of functions that can be passed as arguments:
 
-        1. `Fn(&T)` : the fuzzer will only report a failure when the given function crashes
-        2. `Fn(&T) -> Bool` : the fuzzer will report a failure when the output is `false`
-        3. `Fn(&T) -> Result<_,_>` : the fuzzer will report a failure when the output is `Err(..)`
-    */
-    pub fn test<T, F, TestFunctionKind>(test_function: F) -> FuzzerBuilder1<T, F::NormalizedFunction>
-    where
-        T: ?Sized,
-        F: FuzzTestFunction<T, TestFunctionKind>,
-    {
-        FuzzerBuilder1 {
-            test_function: test_function.test_function(),
-            _phantom: PhantomData,
-        }
+    1. `Fn(&T)` : the fuzzer will only report a failure when the given function crashes
+    2. `Fn(&T) -> Bool` : the fuzzer will report a failure when the output is `false`
+    3. `Fn(&T) -> Result<_,_>` : the fuzzer will report a failure when the output is `Err(..)`
+*/
+#[no_coverage]
+pub fn fuzz_test<T, F, TestFunctionKind>(test_function: F) -> FuzzerBuilder1<T, F::NormalizedFunction>
+where
+    T: ?Sized,
+    F: FuzzTestFunction<T, TestFunctionKind>,
+{
+    FuzzerBuilder1 {
+        test_function: test_function.test_function(),
+        _phantom: PhantomData,
     }
 }
 
+#[cfg(feature = "serde_json_serializer")]
+impl<T, F> FuzzerBuilder1<T, F>
+where
+    T: ?Sized + ToOwned + 'static,
+    T::Owned: Clone + serde::Serialize + for<'e> serde::Deserialize<'e> + DefaultMutator,
+    <T::Owned as DefaultMutator>::Mutator: 'static,
+    F: Fn(&T) -> bool + 'static,
+{
+    #[no_coverage]
+    pub fn default_options(
+        self,
+    ) -> FuzzerBuilder6<
+        T,
+        F,
+        <T::Owned as DefaultMutator>::Mutator,
+        T::Owned,
+        SerdeSerializer<T::Owned>,
+        CodeCoverageSensor,
+        impl Pool<TestCase = FuzzedInput<T::Owned, <T::Owned as DefaultMutator>::Mutator>>
+            + CompatibleWithSensor<CodeCoverageSensor>,
+    > {
+        self.mutator(<T::Owned as DefaultMutator>::default_mutator())
+            .serializer(SerdeSerializer::default())
+            .default_sensor()
+            .default_pool()
+            .arguments_from_cargo_fuzzcheck()
+    }
+}
 impl<T, F> FuzzerBuilder1<T, F>
 where
     T: ?Sized,
@@ -215,7 +227,7 @@ where
         ```
         Then the given mutator should produces values that can be borrowed as `[u8]`.
         We can write:
-        ```
+        ```ignore
         use fuzzcheck::{FuzzerBuilder, DefaultMutator};
         # fn foo(xs: &[u8]) {
         #     // ..
@@ -229,6 +241,7 @@ where
         }
         ```
     */
+    #[no_coverage]
     pub fn mutator<M, V>(self, mutator: M) -> FuzzerBuilder2<T, F, M, V>
     where
         V: Clone + Borrow<T>,
@@ -261,6 +274,7 @@ where
             .serializer(SerdeSerializer::default())
         ```
     */
+    #[no_coverage]
     pub fn serializer<S>(self, serializer: S) -> FuzzerBuilder3<T, F, M, V, S>
     where
         S: Serializer<Value = V>,
@@ -281,10 +295,135 @@ where
     M: Mutator<V>,
     S: Serializer<Value = V>,
 {
-    /**
-        Use the arguments provided to cargo-fuzzcheck when launching this test.
-    */
-    pub fn arguments_from_cargo_fuzzcheck(self) -> FuzzerBuilder4<T, F, M, V, S> {
+    #[no_coverage]
+    pub fn default_sensor(self) -> FuzzerBuilder4<T, F, M, V, S, CodeCoverageSensor> {
+        let sensor = CodeCoverageSensor::observing_only_files_from_current_dir();
+        FuzzerBuilder4 {
+            test_function: self.test_function,
+            mutator: self.mutator,
+            serializer: self.serializer,
+            sensor,
+            _phantom: PhantomData,
+        }
+    }
+    #[no_coverage]
+    pub fn sensor<Sens: Sensor>(self, sensor: Sens) -> FuzzerBuilder4<T, F, M, V, S, Sens> {
+        FuzzerBuilder4 {
+            test_function: self.test_function,
+            mutator: self.mutator,
+            serializer: self.serializer,
+            sensor,
+            _phantom: PhantomData,
+        }
+    }
+}
+
+pub fn default_sensor_and_pool<T: TestCase>() -> (
+    CodeCoverageSensor,
+    impl CompatibleWithSensor<CodeCoverageSensor, TestCase = T>,
+) {
+    let sensor = CodeCoverageSensor::observing_only_files_from_current_dir();
+    let pool = defaul_pool_for_code_coverage_sensor(&sensor);
+    (sensor, pool)
+}
+
+fn defaul_pool_for_code_coverage_sensor<T: TestCase>(
+    sensor: &CodeCoverageSensor,
+) -> impl CompatibleWithSensor<CodeCoverageSensor, TestCase = T> {
+    let count_instrumented = sensor.count_instrumented;
+    let pool = UniqueCoveragePool::new("uniq_cov", count_instrumented);
+    let pool2 = CounterMaximizingPool::new("high_cov_hits", count_instrumented);
+
+    let pool4 = AggregateCoveragePool::<_, SumCounterValues>::new("highest_aggregate_cov_hits");
+    let pool5 = AggregateCoveragePool::<_, CountNumberOfDifferentCounters>::new("most_diverse_cov");
+
+    let pool = AndPool::new(pool2, pool, 1);
+    let pool = AndPool::new(pool, pool4, 254);
+    let pool = AndPool::new(pool, pool5, 254);
+    pool
+}
+
+impl<T, F, M, V, S> FuzzerBuilder4<T, F, M, V, S, CodeCoverageSensor>
+where
+    T: ?Sized,
+    F: Fn(&T) -> bool,
+    V: Clone + Borrow<T>,
+    M: Mutator<V>,
+    S: Serializer<Value = V>,
+{
+    #[no_coverage]
+    pub fn default_pool(
+        self,
+    ) -> FuzzerBuilder5<
+        T,
+        F,
+        M,
+        V,
+        S,
+        CodeCoverageSensor,
+        impl Pool<TestCase = FuzzedInput<V, M>> + CompatibleWithSensor<CodeCoverageSensor>,
+    > {
+        let pool = defaul_pool_for_code_coverage_sensor(&self.sensor);
+        FuzzerBuilder5 {
+            test_function: self.test_function,
+            mutator: self.mutator,
+            serializer: self.serializer,
+            sensor: self.sensor,
+            pool,
+            _phantom: PhantomData,
+        }
+    }
+}
+
+impl<T, F, M, V, S, Sens> FuzzerBuilder4<T, F, M, V, S, Sens>
+where
+    T: ?Sized,
+    F: Fn(&T) -> bool,
+    V: Clone + Borrow<T>,
+    M: Mutator<V>,
+    S: Serializer<Value = V>,
+    Sens: Sensor,
+{
+    #[no_coverage]
+    pub fn pool<P>(self, pool: P) -> FuzzerBuilder5<T, F, M, V, S, Sens, P>
+    where
+        P: Pool<TestCase = FuzzedInput<V, M>> + CompatibleWithSensor<Sens>,
+    {
+        FuzzerBuilder5 {
+            test_function: self.test_function,
+            mutator: self.mutator,
+            serializer: self.serializer,
+            sensor: self.sensor,
+            pool,
+            _phantom: PhantomData,
+        }
+    }
+}
+
+impl<T, F, M, V, S, Sens, P> FuzzerBuilder5<T, F, M, V, S, Sens, P>
+where
+    T: ?Sized,
+    F: Fn(&T) -> bool,
+    V: Clone + Borrow<T>,
+    M: Mutator<V>,
+    S: Serializer<Value = V>,
+    Sens: Sensor,
+    P: Pool<TestCase = FuzzedInput<V, M>> + CompatibleWithSensor<Sens>,
+{
+    #[no_coverage]
+    pub fn arguments(self, arguments: Arguments) -> FuzzerBuilder6<T, F, M, V, S, Sens, P> {
+        FuzzerBuilder6 {
+            test_function: self.test_function,
+            mutator: self.mutator,
+            serializer: self.serializer,
+            sensor: self.sensor,
+            pool: self.pool,
+            arguments,
+            _phantom: self._phantom,
+        }
+    }
+    #[no_coverage]
+    pub fn arguments_from_cargo_fuzzcheck(self) -> FuzzerBuilder6<T, F, M, V, S, Sens, P> {
         let parser = options_parser();
         let mut help = format!(
             r#""
@@ -342,107 +481,98 @@ fuzzcheck {cmin} --{in_corpus} "fuzz-corpus" --{corpus_size} 25
                 std::process::exit(1);
             }
         };
-
-        FuzzerBuilder4 {
+        FuzzerBuilder6 {
             test_function: self.test_function,
             mutator: self.mutator,
             serializer: self.serializer,
+            sensor: self.sensor,
+            pool: self.pool,
             arguments,
             _phantom: PhantomData,
         }
     }
 }
 
-impl<T, F, M, V, S> FuzzerBuilder4<T, F, M, V, S>
+impl<T, F, M, V, S, Sens, P> FuzzerBuilder6<T, F, M, V, S, Sens, P>
 where
     T: ?Sized,
     F: Fn(&T) -> bool,
     V: Clone + Borrow<T>,
     M: Mutator<V>,
     S: Serializer<Value = V>,
-    Fuzzer<V, T, F, M, S>: 'static,
+    Sens: Sensor,
+    P: Pool<TestCase = FuzzedInput<V, M>> + CompatibleWithSensor<Sens>,
+    Fuzzer<V, T, F, M, S, Sens, P>: 'static,
 {
-    /**
-        Only gather code coverage information from files that are children of the directory from which cargo-fuzzcheck is called.
-    */
-    pub fn observe_only_files_from_current_dir(
-        self,
-    ) -> FuzzerBuilder5<T, F, M, V, S, impl Fn(&Path) -> bool, impl Fn(&Path) -> bool> {
-        FuzzerBuilder5 {
-            test_function: self.test_function,
-            mutator: self.mutator,
-            serializer: self.serializer,
-            arguments: self.arguments,
-            exclude: |_| true,
-            keep: |f| f.is_relative(),
-            _phantom: PhantomData,
-        }
+    #[no_coverage]
+    pub fn command(self, command: FuzzerCommand) -> Self {
+        let mut x = self;
+        x.arguments.command = command;
+        x
     }
-
-    /**
-        Specify the files for which code coverage information influences the fuzzer.
-
-        The first argument is the “exclude” closure, of type `Fn(&Path) -> bool`.
-        It takes the path of a file and returns true if code coverage should be ignored
-        for that file. It is essentially a denylist.
-
-        The second argument is the “keep” closure, of type `Fn(&Path) -> bool`.
-        It takes the path of a file and returns true if code coverage should be gathered
-        from that file. It is essentially an allowlist.
-
-        By default, the code coverage from all files is included. The “keep” closure has a
-        higher priority than the “exclude” one.
-    */
-    pub fn observe_files<Exclude, Keep>(
-        self,
-        exclude: Exclude,
-        keep: Keep,
-    ) -> FuzzerBuilder5<T, F, M, V, S, Exclude, Keep>
-    where
-        Exclude: Fn(&Path) -> bool,
-        Keep: Fn(&Path) -> bool,
-    {
-        FuzzerBuilder5 {
-            test_function: self.test_function,
-            mutator: self.mutator,
-            serializer: self.serializer,
-            arguments: self.arguments,
-            exclude,
-            keep,
-            _phantom: PhantomData,
-        }
+    #[no_coverage]
+    pub fn in_corpus(self, path: Option<&Path>) -> Self {
+        let mut x = self;
+        x.arguments.corpus_in = path.map(Path::to_path_buf);
+        x
+    }
+    #[no_coverage]
+    pub fn out_corpus(self, path: Option<&Path>) -> Self {
+        let mut x = self;
+        x.arguments.corpus_out = path.map(Path::to_path_buf);
+        x
+    }
+    #[no_coverage]
+    pub fn artifacts_folder(self, path: Option<&Path>) -> Self {
+        let mut x = self;
+        x.arguments.artifacts_folder = path.map(Path::to_path_buf);
+        x
+    }
+    #[no_coverage]
+    pub fn maximum_complexity(self, max_input_cplx: f64) -> Self {
+        let mut x = self;
+        x.arguments.max_input_cplx = max_input_cplx;
+        x
+    }
+    #[no_coverage]
+    pub fn stop_after_iterations(self, number_of_iterations: usize) -> Self {
+        let mut x = self;
+        x.arguments.maximum_iterations = number_of_iterations;
+        x
+    }
+    #[no_coverage]
+    pub fn stop_after_duration(self, duration: Duration) -> Self {
+        let mut x = self;
+        x.arguments.maximum_duration = duration;
+        x
+    }
+    #[no_coverage]
+    pub fn stop_after_first_test_failure(self, stop_after_first_test_failure: bool) -> Self {
+        let mut x = self;
+        x.arguments.stop_after_first_failure = stop_after_first_test_failure;
+        x
     }
-}
-
-impl<T, F, M, V, S, Exclude, Keep> FuzzerBuilder5<T, F, M, V, S, Exclude, Keep>
-where
-    T: ?Sized,
-    F: Fn(&T) -> bool,
-    V: Clone + Borrow<T>,
-    M: Mutator<V>,
-    S: Serializer<Value = V>,
-    Fuzzer<V, T, F, M, S>: 'static,
-    Exclude: Fn(&Path) -> bool,
-    Keep: Fn(&Path) -> bool,
-{
     /// Launch the fuzz test!
-    pub fn launch(self) {
+    #[no_coverage]
+    pub fn launch(self) -> Result<(), ReasonForStopping<V>> {
         #[cfg(fuzzing)]
-        self.launch_even_if_not_cfg_fuzzing_is_not_set()
+        self.launch_even_if_cfg_fuzzing_is_not_set()?;
+        Ok(())
     }
 
     /// do not use
-    pub fn launch_even_if_not_cfg_fuzzing_is_not_set(self) {
-        let FuzzerBuilder5 {
+    #[no_coverage]
+    pub fn launch_even_if_cfg_fuzzing_is_not_set(self) -> Result<(), ReasonForStopping<V>> {
+        let FuzzerBuilder6 {
             test_function,
             mutator,
             serializer,
+            pool,
+            sensor,
             arguments,
-            exclude,
-            keep,
             _phantom,
         } = self;
 
-        fuzzer::launch(test_function, mutator, exclude, keep, serializer, arguments).unwrap();
+        crate::fuzzer::launch(test_function, mutator, serializer, sensor, pool, arguments)
     }
 }
