@@ -61,6 +61,7 @@ type InnerASTMutator = Either<
 pub struct ASTMutator {
     pub inner: Box<InnerASTMutator>,
 }
+#[derive(Clone)]
 pub struct ASTMutatorCache {
     pub inner: Box<<InnerASTMutator as Mutator<AST>>::Cache>,
 }
@@ -70,6 +71,7 @@ impl ASTMutatorCache {
         Self { inner: Box::new(inner) }
     }
 }
+#[derive(Clone)]
 pub struct ASTMutatorMutationStep {
     pub inner: Box<<InnerASTMutator as Mutator<AST>>::MutationStep>,
 }
@@ -79,6 +81,7 @@ impl ASTMutatorMutationStep {
         Self { inner: Box::new(inner) }
     }
 }
+#[derive(Clone)]
 pub struct ASTMutatorArbitraryStep {
     pub inner: Box<<InnerASTMutator as Mutator<AST>>::ArbitraryStep>,
 }
@@ -165,7 +168,11 @@ impl Mutator<AST> for ASTMutator {
 pub fn grammar_based_string_mutator(grammar: Rc<Grammar>) -> impl Mutator<String> {
     let grammar_cloned = grammar.clone();
     let parse = move |string: &String| parse_from_grammar(string, grammar_cloned.clone());
-    IncrementalMapMutator::<AST, String, ASTMutator, ASTMap, _>::new(parse, ASTMutator::from_grammar(grammar))
+    IncrementalMapMutator::<AST, String, ASTMutator, ASTMap, _>::new(
+        #[no_coverage]
+        parse,
+        ASTMutator::from_grammar(grammar),
+    )
 }
 
 impl ASTMutator {
