@@ -31,10 +31,13 @@ fn grammar_parser<'a>(
             // so we clone it and store it in the parser (that is, in the closure)
             let grammar_long_lived = grammar.clone();
             let mut inner_parser = grammar_parser(string, idx, inner_grammar.clone());
-            Box::new(move || {
-                let _x = grammar_long_lived.as_ref(); // do anything here, to make sure the closure captures grammar_long_lived
-                inner_parser()
-            })
+            Box::new(
+                #[no_coverage]
+                move || {
+                    let _x = grammar_long_lived.as_ref(); // do anything here, to make sure the closure captures grammar_long_lived
+                    inner_parser()
+                },
+            )
         }
         Grammar::Concatenation(gs) => concatenation_parser(string, idx, gs),
     }
@@ -47,7 +50,15 @@ fn recurse_parser<'a>(
     grammar: Rc<Grammar>,
 ) -> Box<dyn 'a + FnMut() -> Option<(AST, usize)>> {
     let mut parser = grammar_parser(string, idx, grammar);
-    Box::new(move || parser().map(|(ast, idx)| (AST::Box(Box::new(ast)), idx)))
+    Box::new(
+        #[no_coverage]
+        move || {
+            parser().map(
+                #[no_coverage]
+                |(ast, idx)| (AST::Box(Box::new(ast)), idx),
+            )
+        },
+    )
 }
 
 #[no_coverage]
@@ -57,24 +68,27 @@ fn atom_parser<'a>(
     range_chars: Range<char>,
 ) -> Box<dyn 'a + FnMut() -> Option<(AST, usize)>> {
     let mut end = false;
-    Box::new(move || {
-        if end || idx >= string.len() {
-            None
-        } else {
-            let string = &string[idx..];
-            let mut chars = string.chars();
-            if let Some(char) = chars.next() {
-                if range_chars.contains(&char) {
-                    end = true;
-                    Some((AST::Token(char), idx + char.len_utf8()))
+    Box::new(
+        #[no_coverage]
+        move || {
+            if end || idx >= string.len() {
+                None
+            } else {
+                let string = &string[idx..];
+                let mut chars = string.chars();
+                if let Some(char) = chars.next() {
+                    if range_chars.contains(&char) {
+                        end = true;
+                        Some((AST::Token(char), idx + char.len_utf8()))
+                    } else {
+                        None
+                    }
                 } else {
                     None
                 }
-            } else {
-                None
             }
-        }
-    })
+        },
+    )
 }
 
 #[no_coverage]
@@ -84,12 +98,15 @@ fn concatenation_parser<'a>(
     gs: &[Rc<Grammar>],
 ) -> Box<dyn 'a + FnMut() -> Option<(AST, usize)>> {
     let mut rec = concatenation_parser_rec(string, idx, Rc::new(List::from_slice(gs)), Rc::new(List::Empty));
-    Box::new(move || {
-        let (asts, idx) = rec()?;
-        let mut asts = asts.to_vec();
-        asts.reverse();
-        Some((AST::Sequence(asts), idx))
-    })
+    Box::new(
+        #[no_coverage]
+        move || {
+            let (asts, idx) = rec()?;
+            let mut asts = asts.to_vec();
+            asts.reverse();
+            Some((AST::Sequence(asts), idx))
+        },
+    )
 }
 
 #[no_coverage]
@@ -102,53 +119,56 @@ fn concatenation_parser_rec<'a>(
     let mut current_grammar_parser: Option<(Box<dyn FnMut() -> Option<(AST, usize)>>, Rc<List<Rc<Grammar>>>)> = None;
     let mut rest_concatenation_parser: Option<Box<dyn FnMut() -> Option<(Rc<List<AST>>, usize)>>> = None;
     let mut last = false;
-    Box::new(move || {
-        if last {
-            return None;
-        }
-        'main: loop {
-            if let Some(rec_parser) = &mut rest_concatenation_parser {
-                if let Some(result) = rec_parser() {
-                    return Some(result);
-                } else {
-                    rest_concatenation_parser = None;
-                    continue 'main;
-                }
-            } else if let Some((grammar_parser, rest)) = &mut current_grammar_parser {
-                if let Some((next_ast, next_idx)) = grammar_parser() {
-                    rest_concatenation_parser = Some(concatenation_parser_rec(
-                        string,
-                        next_idx,
-                        rest.clone(),
-                        current_asts.prepend(next_ast),
-                    ));
-                    continue;
-                } else {
-                    // the grammar doesn't match the string, so the whole concatenation must be invalid
-                    return None;
-                }
-            } else {
-                let deconstructed = match gs.as_ref() {
-                    List::Empty => None,
-                    List::Cons(g, rest) => Some((g.clone(), rest.clone())),
-                };
-                match deconstructed {
-                    None => {
-                        if current_asts.is_empty() {
-                            return None;
-                        } else {
-                            last = true; // next one will return None
-                            return Some((current_asts.clone(), idx));
-                        }
-                    }
-                    Some((r, rest)) => {
-                        current_grammar_parser = Some((grammar_parser(string, idx, r), rest));
+    Box::new(
+        #[no_coverage]
+        move || {
+            if last {
+                return None;
+            }
+            'main: loop {
+                if let Some(rec_parser) = &mut rest_concatenation_parser {
+                    if let Some(result) = rec_parser() {
+                        return Some(result);
+                    } else {
+                        rest_concatenation_parser = None;
                         continue 'main;
                     }
+                } else if let Some((grammar_parser, rest)) = &mut current_grammar_parser {
+                    if let Some((next_ast, next_idx)) = grammar_parser() {
+                        rest_concatenation_parser = Some(concatenation_parser_rec(
+                            string,
+                            next_idx,
+                            rest.clone(),
+                            current_asts.prepend(next_ast),
+                        ));
+                        continue;
+                    } else {
+                        // the grammar doesn't match the string, so the whole concatenation must be invalid
+                        return None;
+                    }
+                } else {
+                    let deconstructed = match gs.as_ref() {
+                        List::Empty => None,
+                        List::Cons(g, rest) => Some((g.clone(), rest.clone())),
+                    };
+                    match deconstructed {
+                        None => {
+                            if current_asts.is_empty() {
+                                return None;
+                            } else {
+                                last = true; // next one will return None
+                                return Some((current_asts.clone(), idx));
+                            }
+                        }
+                        Some((r, rest)) => {
+                            current_grammar_parser = Some((grammar_parser(string, idx, r), rest));
+                            continue 'main;
+                        }
+                    }
                 }
             }
-        }
-    })
+        },
+    )
 }
 
 #[no_coverage]
@@ -159,12 +179,15 @@ fn repetition_parser<'a>(
     range: Range<usize>,
 ) -> Box<dyn 'a + FnMut() -> Option<(AST, usize)>> {
     let mut rec = repetition_parser_rec(string, idx, g, 0, range, Rc::new(List::Empty));
-    Box::new(move || {
-        let (asts, idx) = rec()?;
-        let mut asts = asts.to_vec();
-        asts.reverse();
-        Some((AST::Sequence(asts), idx))
-    })
+    Box::new(
+        #[no_coverage]
+        move || {
+            let (asts, idx) = rec()?;
+            let mut asts = asts.to_vec();
+            asts.reverse();
+            Some((AST::Sequence(asts), idx))
+        },
+    )
 }
 
 #[no_coverage]
@@ -179,43 +202,46 @@ fn repetition_parser_rec<'a>(
     let mut current_grammar_parser: Option<Box<dyn FnMut() -> Option<(AST, usize)>>> = None;
     let mut rest_concatenation_parser: Option<Box<dyn FnMut() -> Option<(Rc<List<AST>>, usize)>>> = None;
     let mut produced_first_output = false;
-    Box::new(move || {
-        'main: loop {
-            if let Some(rec_parser) = &mut rest_concatenation_parser {
-                if let Some(result) = rec_parser() {
-                    return Some(result);
+    Box::new(
+        #[no_coverage]
+        move || {
+            'main: loop {
+                if let Some(rec_parser) = &mut rest_concatenation_parser {
+                    if let Some(result) = rec_parser() {
+                        return Some(result);
+                    } else {
+                        rest_concatenation_parser = None;
+                        continue 'main;
+                    }
+                } else if let Some(grammar_parser) = &mut current_grammar_parser {
+                    if let Some((next_ast, next_idx)) = grammar_parser() {
+                        rest_concatenation_parser = Some(repetition_parser_rec(
+                            string,
+                            next_idx,
+                            g.clone(),
+                            count + 1,
+                            range.clone(),
+                            current_asts.prepend(next_ast),
+                        ));
+                        continue;
+                    } else {
+                        // the grammar doesn't match the string, so the whole concatenation must be invalid
+                        return None;
+                    }
                 } else {
-                    rest_concatenation_parser = None;
+                    if count >= range.end {
+                        return None;
+                    }
+                    if !produced_first_output && range.contains(&count) {
+                        produced_first_output = true;
+                        return Some((current_asts.clone(), idx));
+                    }
+                    current_grammar_parser = Some(grammar_parser(string, idx, g.clone()));
                     continue 'main;
                 }
-            } else if let Some(grammar_parser) = &mut current_grammar_parser {
-                if let Some((next_ast, next_idx)) = grammar_parser() {
-                    rest_concatenation_parser = Some(repetition_parser_rec(
-                        string,
-                        next_idx,
-                        g.clone(),
-                        count + 1,
-                        range.clone(),
-                        current_asts.prepend(next_ast),
-                    ));
-                    continue;
-                } else {
-                    // the grammar doesn't match the string, so the whole concatenation must be invalid
-                    return None;
-                }
-            } else {
-                if count >= range.end {
-                    return None;
-                }
-                if !produced_first_output && range.contains(&count) {
-                    produced_first_output = true;
-                    return Some((current_asts.clone(), idx));
-                }
-                current_grammar_parser = Some(grammar_parser(string, idx, g.clone()));
-                continue 'main;
             }
-        }
-    })
+        },
+    )
 }
 
 #[no_coverage]
@@ -226,23 +252,26 @@ fn alternation_parser<'a>(
 ) -> Box<dyn 'a + FnMut() -> Option<(AST, usize)>> {
     let mut rs_iter = rs.iter();
     let mut parser: Option<Box<dyn FnMut() -> Option<(AST, usize)>>> = None;
-    Box::new(move || 'main: loop {
-        if let Some(p) = &mut parser {
-            if let Some((ast, idx)) = p() {
-                let ast = AST::Box(Box::new(ast));
-                return Some((ast, idx));
-            } else {
-                parser = None;
-                continue 'main;
+    Box::new(
+        #[no_coverage]
+        move || 'main: loop {
+            if let Some(p) = &mut parser {
+                if let Some((ast, idx)) = p() {
+                    let ast = AST::Box(Box::new(ast));
+                    return Some((ast, idx));
+                } else {
+                    parser = None;
+                    continue 'main;
+                }
             }
-        }
-        if let Some(grammar) = rs_iter.next() {
-            parser = Some(grammar_parser(string, idx, grammar));
-            continue 'main;
-        } else {
-            return None;
-        }
-    })
+            if let Some(grammar) = rs_iter.next() {
+                parser = Some(grammar_parser(string, idx, grammar));
+                continue 'main;
+            } else {
+                return None;
+            }
+        },
+    )
 }
 
 // #[no_coverage] fn parse_end<'a>(string: &'a str, idx: usize) -> Box<dyn 'a + FnMut() -> Option<(AST, usize)>> {
